import { Server, Socket } from 'socket.io';

/**
 * Type alias for a constructor of a Room.
 * @template T The constructor parameters type.
 */
type RoomConstructor<T extends unknown[] = unknown[]> = new (...args: T) => Room;
/**
 * Type for room registration, holding the room type and its constructor.
 * @template RT The room type.
 * @template T The constructor parameters type.
 */
type RoomRegistration<RT, T extends unknown[] = unknown[]> = {
    type: RT;
    constructor: RoomConstructor<T>;
};
/**
 * Represents a server with the ability to manage different types of rooms.
 * @template RT The room type.
 */
declare class CyberCafeServer<RT> {
    io: Server;
    private rooms;
    private roomConstructors;
    /**
     * Initializes a new instance of the CyberCafeServer class.
     */
    constructor(io: Server);
    /**
     * Defines a new room type and its associated constructor.
     * @param registration The room registration object containing the room type and constructor.
     */
    defineRoomType<T extends unknown[]>(registration: RoomRegistration<RT, T>): void;
    /**
     * Creates a new room of the specified type.
     * @param roomType The type of the room to create.
     * @param args The constructor arguments for the room.
     * @returns The newly created room instance.
     * @throws When the room type is unrecognized or the room ID is undefined.
     */
    createRoom<T extends string[]>(roomType: RT, ...args: T): Room;
    /**
     * Retrieves a room by its ID.
     * @param roomId The ID of the room to retrieve.
     * @returns The room instance if found, otherwise undefined.
     */
    getRoom(roomId: string): Room | undefined;
    /**
     * Removes a room from the server by its ID.
     * @param roomId The ID of the room to remove.
     */
    removeRoom(roomId: string): void;
}

/**
 * Represents a room in the cybercafe server.
 */
declare class Room {
    private namespace;
    users: Map<string, Socket>;
    private state;
    private server;
    /**
     * Initializes a new instance of the Room class.
     * @param namespace The namespace identifier for the room.
     */
    constructor(namespace: string, server: CyberCafeServer<string>);
    /**
     * Updates the room state with the provided state object.
     * @param update A partial state object with properties to update.
     */
    updateState(update: Record<string, unknown>): void;
    /**
     * Registers an event handler for a specific event.
     * @param event The name of the event to listen for.
     * @param handler The function to call when the event is emitted.
     */
    on(event: string, handler: (socket: Socket, data: unknown) => void): void;
    /**
     * Called when the room is created. Can be overridden by subclasses.
     */
    protected onCreate(): void;
    /**
     * Called when a user joins the room. Can be overridden by subclasses.
     */
    protected onJoin(): void;
    /**
     * Adds a user to the room.
     * @param userId The unique identifier for the user.
     * @param socket The socket instance for the user.
     */
    addUser(userId: string, socket: Socket): void;
    /**
     * Removes a user from the room.
     * @param userId The unique identifier for the user to remove.
     */
    removeUser(userId: string): void;
    /**
     * Broadcasts an event to all users in the room.
     * @param event The name of the event to broadcast.
     * @param data The data to send with the event.
     */
    broadcast(event: string, data: unknown): void;
}

export { CyberCafeServer, Room };
