"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  CyberCafeServer: () => CyberCafeServer,
  Room: () => Room
});
module.exports = __toCommonJS(src_exports);

// src/classes/Room.ts
var Room = class {
  namespace;
  users;
  state;
  server;
  /**
   * Initializes a new instance of the Room class.
   * @param namespace The namespace identifier for the room.
   */
  constructor(namespace, server) {
    this.state = {};
    this.server = server;
    this.namespace = this.server.io.of(namespace);
    this.users = /* @__PURE__ */ new Map();
    this.onCreate();
    this.namespace.on("connection", (socket) => {
      socket.emit("stateUpdate", this.state);
      this.onJoin();
    });
  }
  /**
   * Updates the room state with the provided state object.
   * @param update A partial state object with properties to update.
   */
  updateState(update) {
    this.state = { ...this.state, ...update };
    this.namespace.emit("stateUpdate", this.state);
  }
  /**
   * Registers an event handler for a specific event.
   * @param event The name of the event to listen for.
   * @param handler The function to call when the event is emitted.
   */
  on(event, handler) {
    this.namespace.on(event, handler);
  }
  /**
   * Called when the room is created. Can be overridden by subclasses.
   */
  onCreate() {
  }
  /**
   * Called when a user joins the room. Can be overridden by subclasses.
   */
  onJoin() {
  }
  /**
   * Adds a user to the room.
   * @param userId The unique identifier for the user.
   * @param socket The socket instance for the user.
   */
  addUser(userId, socket) {
    this.users.set(userId, socket);
    socket.join(this.namespace.name);
  }
  /**
   * Removes a user from the room.
   * @param userId The unique identifier for the user to remove.
   */
  removeUser(userId) {
    const user = this.users.get(userId);
    if (user) {
      user.leave(this.namespace.name);
      this.users.delete(userId);
    }
  }
  /**
   * Broadcasts an event to all users in the room.
   * @param event The name of the event to broadcast.
   * @param data The data to send with the event.
   */
  broadcast(event, data) {
    this.namespace.emit(event, data);
  }
};

// src/classes/Server.ts
var CyberCafeServer = class {
  io;
  rooms;
  roomConstructors;
  /**
   * Initializes a new instance of the CyberCafeServer class.
   */
  constructor(io) {
    this.io = io;
    this.rooms = /* @__PURE__ */ new Map();
    this.roomConstructors = /* @__PURE__ */ new Map();
  }
  /**
   * Defines a new room type and its associated constructor.
   * @param registration The room registration object containing the room type and constructor.
   */
  defineRoomType(registration) {
    this.roomConstructors.set(
      registration.type,
      registration.constructor
    );
  }
  /**
   * Creates a new room of the specified type.
   * @param roomType The type of the room to create.
   * @param args The constructor arguments for the room.
   * @returns The newly created room instance.
   * @throws When the room type is unrecognized or the room ID is undefined.
   */
  createRoom(roomType, ...args) {
    const Constructor = this.roomConstructors.get(roomType);
    if (Constructor) {
      const room = new Constructor(...args, this);
      const roomId = args[0];
      if (roomId === void 0) {
        throw new Error("Room ID cannot be undefined");
      }
      this.rooms.set(roomId, room);
      return room;
    } else {
      throw new Error("Unrecognized room type");
    }
  }
  /**
   * Retrieves a room by its ID.
   * @param roomId The ID of the room to retrieve.
   * @returns The room instance if found, otherwise undefined.
   */
  getRoom(roomId) {
    return this.rooms.get(roomId);
  }
  /**
   * Removes a room from the server by its ID.
   * @param roomId The ID of the room to remove.
   */
  removeRoom(roomId) {
    this.rooms.delete(roomId);
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  CyberCafeServer,
  Room
});
